# Copyright 2011  Lars Wirzenius
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


__version__ = '0.22'


import os


def cat(filename):
    '''Return contents of file, or empty string if it doesn't exist.'''
    if os.path.exists(filename):
        with open(filename) as f:
            return f.read()
    else:
        return ''


class TestCase(object):

    def __init__(self, name, path_prefix):
        self.name = name
        self.path_prefix = path_prefix


class TestDir(object):

    '''Contain information about a directory of test cases.'''
    
    per_test_suffixes = ('script', 'stdin', 'stdout', 'stderr', 
                         'exit', 'setup', 'teardown')
    
    def __init__(self):
        self.setup = None
        self.setup_once = None
        self.tests = []
        self.teardown = None
        self.teardown_once = None
    
    def scan(self, dirname, filenames=None):
        filenames = os.listdir(dirname) if filenames is None else filenames

        script_names = ['setup_once', 'setup', 'teardown', 'teardown_once']
        for name in script_names:
            if name in filenames:
                full_path = os.path.join(dirname, name)
                setattr(self, name, full_path)

        prefixes = self.find_prefixes(filenames)
        for prefix in prefixes:
            test = TestCase(prefix, os.path.join(dirname, prefix))
            for suffix in self.per_test_suffixes:
                name = '%s.%s' % (prefix, suffix)
                if name in filenames:
                    value = os.path.join(dirname, name)
                else:
                    value = None
                setattr(test, suffix, value)
            self.tests.append(test)
            
    def find_prefixes(self, filenames):
        prefixes = set()
        for filename in filenames:
            for suffix in self.per_test_suffixes:
                suffix = '.' + suffix
                if filename.endswith(suffix):
                    prefix = filename[:-len(suffix)]
                    prefixes.add(prefix)
        return sorted(list(prefixes))




# Copyright 2013  Lars Wirzenius
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# =*= License: GPL-3+ =*=


# This is a step in a scenario: GIVEN, WHEN, THEN, etc.

class ScenarioStep(object):

    def __init__(self, what, text):
        self.what = what
        self.text = text
        self.implementation = None


# This is the scenario itself.

class Scenario(object):

    def __init__(self, name):
        self.name = name
        self.steps = []


# This is an IMPLEMENTS chunk.

class Implementation(object):

    def __init__(self, what, regexp, shell):
        self.what = what
        self.regexp = regexp
        self.shell = shell




# Copyright 2011  Lars Wirzenius
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import shutil
import tempfile
import unittest

from cmdtestlib import TestDir, cat


class TestDirTests(unittest.TestCase):

    def test_finds_nothing_for_empty_directory(self):
        td = TestDir()
        td.scan('tests', filenames=[])
        self.assertEqual(td.setup_once, None)
        self.assertEqual(td.setup, None)
        self.assertEqual(td.tests, [])
        self.assertEqual(td.teardown, None)
        self.assertEqual(td.teardown_once, None)

    def test_finds_setup_and_teardown_files(self):
        td = TestDir()
        td.scan('tests', filenames=['setup_once', 'setup', 'teardown', 
                                    'teardown_once'])
        self.assertEqual(td.setup_once, 'tests/setup_once')
        self.assertEqual(td.setup, 'tests/setup')
        self.assertEqual(td.tests, [])
        self.assertEqual(td.teardown, 'tests/teardown')
        self.assertEqual(td.teardown_once, 'tests/teardown_once')

    def test_finds_tests(self):
        td = TestDir()
        td.scan('tests', filenames=['foo.script'])
        self.assertEqual(len(td.tests), 1)
        test = td.tests[0]
        self.assertEqual(test.name, 'foo')
        self.assertEqual(test.script, 'tests/foo.script')
        self.assertEqual(test.stdin, None)

    def test_finds_no_prefixes_when_there_are_none(self):
        td = TestDir()
        self.assertEqual(td.find_prefixes(['setup']), [])

    def test_finds_single_prefix(self):
        td = TestDir()
        self.assertEqual(td.find_prefixes(['setup', 'foo.setup']), ['foo'])

    def test_finds_two_prefixes(self):
        td = TestDir()
        self.assertEqual(td.find_prefixes(['setup', 'foo.setup', 'bar.script']), 
                         ['bar', 'foo'])


class CatTests(unittest.TestCase):

    def setUp(self):
        self.tempdir = tempfile.mkdtemp()
        
    def tearDown(self):
        shutil.rmtree(self.tempdir)
        
    def test_returns_empty_string_for_nonexistent_file(self):
        filename = os.path.join(self.tempdir, 'file.txt')
        self.assertEqual(cat(filename), '')
        
    def test_returns_contents_of_file(self):
        filename = os.path.join(self.tempdir, 'file.txt')
        with open(filename, 'w') as f:
            f.write('foobar')
        self.assertEqual(cat(filename), 'foobar')




# Copyright 2013  Lars Wirzenius
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# =*= License: GPL-3+ =*=


import logging
import markdown
import StringIO
from markdown.treeprocessors import Treeprocessor


#
# Classes for Markdown parsing. See python-markdown documentation
# for details. We want to find all top level code blocks (indented
# four spaces in the Markdown), which we'll parse for scenario test
# stuff later on. We create a Python markdown extension and use
# "tree processor" to analyse the parsed ElementTree at the right
# moment for top level <pre> blocks.
#

# This is a Treeprocessor that iterates over the parsed Markdown,
# as an ElementTree, and finds all top level code blocks.

class GatherCodeBlocks(Treeprocessor):

    def __init__(self, blocks):
        self.blocks = blocks

    def run(self, root):
        for child in root.getchildren():
            if child.tag == 'pre':
                code = child.find('code')
                self.blocks.append(code.text)
        return root

# This is the Python Markdown extension to call the code block
# gatherer at the right time. It stores the list of top level
# code blocks as the blocks attribute.

class ParseScenarioTestBlocks(markdown.extensions.Extension):

    def extendMarkdown(self, md, md_globals):
        self.blocks = []
        self.gatherer = GatherCodeBlocks(self.blocks)
        md.treeprocessors.add('gathercode', self.gatherer, '_end')


class MarkdownParser(object):

    def __init__(self):
        self.blocks = []

    def parse_string(self, text):
        ext = ParseScenarioTestBlocks()
        f = StringIO.StringIO()
        markdown.markdown(text, output=f, extensions=[ext])
        self.blocks.extend(ext.blocks)
        return ext.blocks

    def parse_file(self, filename): # pragma: no cover
        with open(filename) as f:
            binary = f.read()
            text = binary.decode('utf-8')
            return self.parse_string(text)



# Copyright 2013  Lars Wirzenius
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# =*= License: GPL-3+ =*=


import cliapp

import yarnlib


class BlockError(cliapp.AppException):

    pass


# Parse a sequence of textual blocks into scenario and Implementation
# objects, and their constituent objects.

class BlockParser(object):

    def __init__(self):
        self.scenarios = []
        self.implementations = []
        self.line_parsers = {
            'SCENARIO': self.parse_scenario,
            'ASSUMING': self.parse_assuming,
            'GIVEN': self.parse_given,
            'WHEN': self.parse_when,
            'THEN': self.parse_then,
            'FINALLY': self.parse_finally,
            'AND': self.parse_and,
            '...': self.parse_continuation,
            'IMPLEMENTS': self.parse_implementing,
            'EXAMPLE': self.parse_example,
        }

    def parse_blocks(self, blocks):
        while blocks:
            blocks = self.parse_one(blocks)

    def parse_one(self, blocks):
        assert blocks
        block = blocks[0]
        assert block
        t = block.split('\n', 1)
        assert len(t) in [1,2]
        if len(t) == 1:
            line1 = block
            block = ''
        else:
            line1, block = t
        if block:
            blocks[0] = block
        else:
            del blocks[0]

        words = line1.split()
        if not words:
            return blocks
        rest = ' '.join(words[1:])

        for keyword in self.line_parsers:
            if words[0] == keyword:
                return self.line_parsers[keyword](rest, blocks)

        raise BlockError("Syntax error: unknown step: %s" % line1)

    def parse_scenario(self, line, blocks):
        self.scenarios.append(yarnlib.Scenario(line))
        return blocks

    def parse_simple(self, what, line, blocks):
        if not self.scenarios:
            raise BlockError('Syntax errror: %s before SCENARIO' % what)
        step = yarnlib.ScenarioStep(what, line)
        self.scenarios[-1].steps.append(step)
        return blocks

    def parse_assuming(self, line, blocks):
        return self.parse_simple('ASSUMING', line, blocks)

    def parse_given(self, line, blocks):
        return self.parse_simple('GIVEN', line, blocks)

    def parse_when(self, line, blocks):
        return self.parse_simple('WHEN', line, blocks)

    def parse_then(self, line, blocks):
        return self.parse_simple('THEN', line, blocks)

    def parse_finally(self, line, blocks):
        return self.parse_simple('FINALLY', line, blocks)

    def parse_and(self, line, blocks):
        if not self.scenarios:
            raise BlockError('Syntax errror: AND before SCENARIO')
        scenario = self.scenarios[-1]
        if not scenario.steps:
            raise BlockError(
                'Syntax errror: AND before what it would continue')
        step = scenario.steps[-1]
        assert step.what in self.line_parsers
        return self.line_parsers[step.what](line, blocks)

    def parse_continuation(self, line, blocks):
        scenario = self.scenarios[-1]
        step = scenario.steps[-1]
        text = '%s %s' % (step.text, line)
        del scenario.steps[-1]
        return self.line_parsers[step.what](text, blocks)

    def parse_implementing(self, line, blocks):
        words = line.split()
        if len(words) < 2:
            raise BlockError(
                'Syntax error: IMPLEMENTS must have what and regexp')
        what = words[0]
        regexp = ' '.join(words[1:])
        if blocks:
            block = blocks[0]
            shell = []
            rest = []
            for block_line in block.splitlines():
                if rest or block_line.startswith('IMPLEMENTS'):
                    rest.append(block_line)
                else:
                    shell.append(block_line)
            shell = '\n'.join(shell)
            if rest:
                blocks[0] = '\n'.join(rest)
            else:
                del blocks[0]
        else:
            shell = ''
        implementation = yarnlib.Implementation(what, regexp, shell)
        self.implementations.append(implementation)
        return blocks

    def parse_example(self, line, blocks):
        if blocks:
            del blocks[0]
        return blocks



# Copyright 2013  Lars Wirzenius
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# =*= License: GPL-3+ =*=


import unittest

import yarnlib


class MarkdownParserTests(unittest.TestCase):

    def setUp(self):
        self.parser = yarnlib.MarkdownParser()

    def test_finds_code_block(self):
        result = self.parser.parse_string('''
This is blah blah text.

    this is a code block

More text.
''')
        self.assertEqual(self.parser.blocks, ['this is a code block\n'])
        self.assertEqual(result, ['this is a code block\n'])

    def test_finds_consecutive_code_blocks_as_one(self):
        self.parser.parse_string('''
This is blah blah text.

    this is a code block

    this is a second code block

More text.
''')
        self.assertEqual(
            self.parser.blocks,
            ['this is a code block\n\nthis is a second code block\n'])

    def test_finds_code_blocks_with_text_in_between_as_two_blocks(self):
        self.parser.parse_string('''
This is blah blah text.

    this is a code block

Blah.

    this is a second code block

More text.
''')
        self.assertEqual(
            self.parser.blocks,
            ['this is a code block\n', 'this is a second code block\n'])

    def test_only_finds_top_level_code_blocks(self):
        self.parser.parse_string('''
This is blah blah text.

    this is a code block

And now a list:

*   list item

        this is a second level code block

More text.
''')
        self.assertEqual(self.parser.blocks, ['this is a code block\n'])

    def test_parses_multiple_files(self):
        result1 = self.parser.parse_string('''
    block 1
''')
        result2 = self.parser.parse_string('''
    block 2
''')
        self.assertEqual(result1, ['block 1\n'])
        self.assertEqual(result2, ['block 2\n'])
        self.assertEqual(self.parser.blocks, ['block 1\n', 'block 2\n'])




# Copyright 2013  Lars Wirzenius
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# =*= License: GPL-3+ =*=


import unittest

import yarnlib


class BlockParserTests(unittest.TestCase):

    def setUp(self):
        self.parser = yarnlib.BlockParser()

    def test_is_initially_empty(self):
        self.assertEqual(self.parser.scenarios, [])
        self.assertEqual(self.parser.implementations, [])

    def test_skips_examples(self):
        self.parser.parse_blocks(['EXAMPLE foo', 'bar'])
        self.assertEqual(len(self.parser.scenarios), 0)

    def test_parses_simple_elements(self):
        self.parser.parse_blocks(
            ['SCENARIO foo', 'ASSUMING something', 'GIVEN bar',
             'WHEN foobar\nTHEN yoyo\nFINALLY yay\nAND yeehaa'])

        self.assertEqual(len(self.parser.scenarios), 1)
        self.assertEqual(len(self.parser.implementations), 0)

        scenario = self.parser.scenarios[0]
        self.assertEqual(scenario.name, 'foo')
        self.assertEqual(len(scenario.steps), 6)
        self.assertEqual(scenario.steps[0].what, 'ASSUMING')
        self.assertEqual(scenario.steps[0].text, 'something')
        self.assertEqual(scenario.steps[1].what, 'GIVEN')
        self.assertEqual(scenario.steps[1].text, 'bar')
        self.assertEqual(scenario.steps[2].what, 'WHEN')
        self.assertEqual(scenario.steps[2].text, 'foobar')
        self.assertEqual(scenario.steps[3].what, 'THEN')
        self.assertEqual(scenario.steps[3].text, 'yoyo')
        self.assertEqual(scenario.steps[4].what, 'FINALLY')
        self.assertEqual(scenario.steps[4].text, 'yay')
        self.assertEqual(scenario.steps[5].what, 'FINALLY')
        self.assertEqual(scenario.steps[5].text, 'yeehaa')

    def test_handles_continuation_line(self):
        self.parser.parse_blocks(['SCENARIO foo', 'GIVEN foo', '... and bar'])
        scenario = self.parser.scenarios[0]
        self.assertEqual(len(self.parser.scenarios), 1)
        self.assertEqual(scenario.name, 'foo')
        self.assertEqual(scenario.steps[0].what, 'GIVEN')
        self.assertEqual(scenario.steps[0].text, 'foo and bar')

    def test_normalises_whitespace(self):
        self.parser.parse_blocks(['SCENARIO   foo   bar   '])
        self.assertEqual(self.parser.scenarios[0].name, 'foo bar')

    def test_handles_empty_line(self):
        self.parser.parse_blocks(['SCENARIO foo\n\nGIVEN bar\nTHEN foobar'])
        self.assertEqual(len(self.parser.scenarios), 1)

    def test_raises_error_for_unknown_step(self):
        self.assertRaises(
            yarnlib.BlockError,
            self.parser.parse_blocks,
            ['SCENARIO foo\nblah'])

    def test_raises_error_for_step_outside_scenario(self):
        self.assertRaises(
            yarnlib.BlockError,
            self.parser.parse_blocks,
            ['GIVEN foo'])

    def test_raises_error_for_AND_before_scenario(self):
        self.assertRaises(
            yarnlib.BlockError,
            self.parser.parse_blocks,
            ['AND bar'])

    def test_raises_error_for_AND_before_step(self):
        self.assertRaises(
            yarnlib.BlockError,
            self.parser.parse_blocks,
            ['SCENARIO foo\nAND bar'])

    def test_parses_implements_in_a_block_by_itself(self):
        self.parser.parse_blocks(['IMPLEMENTS GIVEN foo\ntrue'])
        impls = self.parser.implementations
        self.assertEqual(len(impls), 1)
        self.assertEqual(impls[0].what, 'GIVEN')
        self.assertEqual(impls[0].regexp, 'foo')
        self.assertEqual(impls[0].shell, 'true')

    def test_parses_implements_with_empty_shell_text(self):
        self.parser.parse_blocks(['IMPLEMENTS GIVEN foo'])
        impls = self.parser.implementations
        self.assertEqual(len(impls), 1)
        self.assertEqual(impls[0].what, 'GIVEN')
        self.assertEqual(impls[0].regexp, 'foo')
        self.assertEqual(impls[0].shell, '')

    def test_parses_two_implements_in_a_code_block(self):
        self.parser.parse_blocks(
            ['IMPLEMENTS GIVEN foo\ntrue\nIMPLEMENTS WHEN bar\ncat /dev/null'])
        impls = self.parser.implementations
        self.assertEqual(len(impls), 2)
        self.assertEqual(impls[0].what, 'GIVEN')
        self.assertEqual(impls[0].regexp, 'foo')
        self.assertEqual(impls[0].shell, 'true')
        self.assertEqual(impls[1].what, 'WHEN')
        self.assertEqual(impls[1].regexp, 'bar')
        self.assertEqual(impls[1].shell, 'cat /dev/null')

    def test_raises_error_for_implements_with_no_args(self):
        self.assertRaises(
            yarnlib.BlockError,
            self.parser.parse_blocks,
            ['IMPLEMENTS'])

    def test_raises_error_for_implements_with_one_args(self):
        self.assertRaises(
            yarnlib.BlockError,
            self.parser.parse_blocks,
            ['IMPLEMENTS GIVEN'])

    def test_raises_error_for_implements_with_first_args_not_a_keyword(self):
        self.assertRaises(
            yarnlib.BlockError,
            self.parser.parse_blocks,
            ['IMPLEMENTS foo'])




#!/usr/bin/python
# Copyright (C) 2011  Lars Wirzenius <liw@liw.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from distutils.core import setup, Extension
from distutils.cmd import Command
from distutils.command.build import build
from distutils.command.clean import clean
import glob
import os
import shutil
import subprocess

import cmdtestlib

try:
    import markdown
except ImportError:
    markdown_version = None
else:
    if (hasattr(markdown, 'extensions') and
        hasattr(markdown.extensions, 'Extension')):
        markdown_version = True
    else:
        markdown_version = False


class GenerateManpage(build):

    def run(self):
        build.run(self)
        print 'building manpages'
        cmds = ['cmdtest']
        if markdown_version:
            cmds.append('yarn')
        for x in cmds:
            with open('%s.1' % x, 'w') as f:
                subprocess.check_call(['python', x,
                                       '--generate-manpage=%s.1.in' % x,
                                       '--output=%s.1' % x], stdout=f)


class CleanMore(clean):

    def run(self):
        clean.run(self)
        for x in ['.coverage', 'cmdtest.1'] + glob.glob('*.pyc'):
            if os.path.exists(x):
                os.remove(x)


class Check(Command):

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        if markdown_version:
            subprocess.check_call(
                ['python', '-m', 'CoverageTestRunner',
                 '--ignore-missing-from', 'without-tests'])
            if os.path.exists('.coverage'):
                os.remove('.coverage')

        subprocess.check_call(['./cmdtest', 'echo-tests'])
        subprocess.check_call(['./cmdtest', 'sort-tests'])

        try:
            subprocess.check_call(['./cmdtest', 'fail-tests'])
        except subprocess.CalledProcessError:
            pass
        else:
            raise Exception('fail-tests did not fail, which is a surprise')

        if markdown_version:
            subprocess.check_call(['./cmdtest', 'yarn.tests'])


setup(name='cmdtest',
      version=cmdtestlib.__version__,
      description='blackbox testing of Unix command line tools',
      author='Lars Wirzenius',
      author_email='liw@liw.fi',
      url='http://liw.fi/cmdtest/',
      scripts=['cmdtest', 'yarn'],
      py_modules=['cmdtestlib'],
      packages=['yarnlib'],
      data_files=[('share/man/man1', glob.glob('*.1'))],
      cmdclass={
        'build': GenerateManpage,
        'check': Check,
        'clean': CleanMore,
      },
     )
